use strict;

use Test::More;
use DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );

sub TestBuild {
    ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
    my $Builder = DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder->new();

    my $hParams = $Builder->addBooleanFilter( 'IsNew', 1 )
        ->addNumericRangeFilter( 'Price', 10.0, 20.0 )
        ->addSimpleFilter( 'FilterKey', 'FilterValue' )
        ->lang('de')
        ->query('  ') # testing empty query
        ->currency('EUR')
        ->sortBy('RELEVANCE')
        ->currentPage(1)
        ->params()
    ;

    is_deeply( $hParams, {
        'IsNew' => 1,
        'Price' => {
            'min' => 10.0,
            'max' => 20.0,
        },
        'FilterKey' => 'FilterValue',
        'lang' => 'de',
        'currency' => 'EUR',
        'sortBy' => 'RELEVANCE',
        'currentPage' => 1,
    }, 'built searchparams' );
    
    $Builder->sortBy('WrongSortKey');
    $hParams = $Builder->params();
    is($hParams->{sortBy}, 'RELEVANCE', 'wrong sortBy key default to RELEVANCE');
}

sub TestEdgeCases {
    ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
    my $Builder = DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder->new();

    eval { $Builder->addNumericRangeFilter( 'OutOfOrder', 25.0, 20.0 ); };
    ok( !ExistsError, 'addNumericRangeFilter' ) or diag( GetError->message );
    is( $Builder->params()->{OutOfOrder}->{min}, 20, 'addNumericRangeFilter - order min max' );

    eval { $Builder->addNumericRangeFilter( 'HalfOpen', 22.0, undef ); };
    ok( !ExistsError, 'addNumericRangeFilter' ) or diag( GetError->message );
    is( $Builder->params()->{HalfOpen}->{min}, 22,    'addNumericRangeFilter - min is set' );
    is( $Builder->params()->{HalfOpen}->{max}, undef, 'addNumericRangeFilter - max is undef' );

}

RunScript(
    Sub => sub {
        eval {
            TestBuild();
            TestEdgeCases();
        };
        if (ExistsError()) {
            my $Error = GetError();
            fail($Error->longMessage);
        }
        done_testing();
    }
);
